part of play;

class PlayVideoPlayer {
  late dart_vlc.Player desktopPlayer;
  late pod_player.PodPlayerController mobilePlayer;
  late bool isInit = false;
  PlayVideoPlayer();

  Future<void> initialize() async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer = dart_vlc.Player(id: 0, videoDimensions: const dart_vlc.VideoDimensions(640, 460), registerTexture: !Platform.isWindows);
      isInit = true;
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = pod_player.PodPlayerController(
        playVideoFrom: pod_player.PlayVideoFrom.youtube(
          'https://www.youtube.com/watch?v=bQowlkLQzkg',
        ),
      );
      await mobilePlayer.initialise();
      if (mobilePlayer.isVideoPlaying) {
        mobilePlayer.pause();
      }
      isInit = true;
    }
  }

  test() async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(
              File('/home/hexaminate/Downloads/y2mate.com - NMIXX OO Performance Video_1080p.mp4'),
            ),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = pod_player.PodPlayerController(
        playVideoFrom: pod_player.PlayVideoFrom.youtube(
          'https://www.youtube.com/watch?v=bQowlkLQzkg',
        ),
      );
      await mobilePlayer.initialise();
    }
  }

  void playFromFile(File file) {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    }
  }

  FutureOr<bool> openYoutube(String url) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.network(url),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = pod_player.PodPlayerController(
        playVideoFrom: pod_player.PlayVideoFrom.youtube(
          url,
        ),
      );
      await mobilePlayer.initialise();
    }
    return false;
  }

  FutureOr<bool> openNetwork(String url) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.network(url),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = pod_player.PodPlayerController(
        playVideoFrom: pod_player.PlayVideoFrom.network(
          url,
        ),
      );
      await mobilePlayer.initialise();
    }
    return false;
  }

  FutureOr<bool> openFile(File file) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = pod_player.PodPlayerController(
        playVideoFrom: pod_player.PlayVideoFrom.file(
          file,
        ),
      );
      await mobilePlayer.initialise();
    }
    return false;
  }

  FutureOr<bool> changeYoutube(String url) async {
    if (Platform.isWindows || Platform.isLinux) {
      // desktopPlayer.open(
      //   dart_vlc.Playlist(
      //     medias: [
      //       dart_vlc.Media.file(file),
      //     ],
      //     playlistMode: dart_vlc.PlaylistMode.single,
      //   ),
      // );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.changeVideo(
        playVideoFrom: pod_player.PlayVideoFrom.youtube(
          url,
        ),
      );
      await mobilePlayer.initialise();
    }
    return false;
  }

  FutureOr<bool> changeFile(File file) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.changeVideo(
        playVideoFrom: pod_player.PlayVideoFrom.file(
          file,
        ),
      );
      await mobilePlayer.initialise();
    }
    return false;
  }

  Duration getDurationMax() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.position.duration ?? Duration();
    }
    return Duration();
  }

  Duration getDurationPosition() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.position.position ?? Duration();
    }
    return Duration();
  }

  streamDurationPosition() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.positionStream;
    } else if (Platform.isAndroid || Platform.isIOS) {}
    return Stream.value("ok");
  }

  seek(Duration duration) {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.seek(duration);
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.videoSeekTo(duration);
    }
  }

  playOrPause() {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.playOrPause();
    }
    if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.isVideoPlaying ? mobilePlayer.pause() : mobilePlayer.play();
    }
  }
}

class PlayVideoWidget extends StatefulWidget {
  final double height;
  final double width;
  final double videoHeight;
  final double videoWidth;
  final Duration videoDurationMax;
  final Duration videoDurationPosition;
  final void Function(double newValue)? onChangedDuration;
  PlayVideoPlayer playVideoPlayer;
  final bool isShowControll;
  PlayVideoWidget({
    Key? key,
    required this.playVideoPlayer,
    required this.height,
    required this.width,
    required this.videoHeight,
    required this.videoWidth,
    required this.videoDurationMax,
    required this.videoDurationPosition,
    required this.onChangedDuration,
    this.isShowControll = true,
  }) : super(key: key);

  @override
  State<PlayVideoWidget> createState() => _PlayVideoWidgetState();
}

class _PlayVideoWidgetState extends State<PlayVideoWidget> {
  @override
  void initState() {
    super.initState();
    taskVideo();
  }

  taskVideo() async {
    await widget.playVideoPlayer.initialize();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    late Widget videoWidget = Container();
    if (widget.playVideoPlayer.isInit) {
      if (Platform.isWindows || Platform.isLinux) {
        videoWidget = dart_vlc.Video(
          player: widget.playVideoPlayer.desktopPlayer,
          width: widget.videoHeight,
          height: widget.videoWidth,
          volumeThumbColor: Colors.blue,
          volumeActiveColor: Colors.blue,
          showControls: false,
          showFullscreenButton: true,
          showTimeLeft: true,
          fillColor: Colors.black,
        );
      } else if (Platform.isAndroid || Platform.isIOS) {
        videoWidget = pod_player.PodVideoPlayer(controller: widget.playVideoPlayer.mobilePlayer);
      }
    }

    return frame(videoWidget);
  }

  Widget frame(Widget child) {
    if (Platform.isWindows || Platform.isLinux) {
      return Container(
        height: widget.height,
        width: widget.width,
        decoration: BoxDecoration(
          color: Colors.black,
          borderRadius: BorderRadius.all(
            Radius.circular(20),
          ),
        ),
        child: Stack(
          fit: StackFit.passthrough,
          children: [
            child,
            Positioned(
              bottom: 5,
              left: 0,
              right: 0,
              child: Padding(
                padding: const EdgeInsets.symmetric(horizontal: 20),
                child: Row(
                  children: [
                    InkWell(
                      onTap: () {
                        widget.playVideoPlayer.playOrPause();
                        setState(() {});
                      },
                      child: const Icon(
                        Icons.pause,
                        color: Colors.white,
                      ),
                    ),
                    InkWell(
                      onTap: () {},
                      child: const Icon(
                        Icons.skip_next,
                        color: Colors.white,
                      ),
                    ),
                    Expanded(
                      child: StreamBuilder(
                        stream: widget.playVideoPlayer.streamDurationPosition(),
                        builder: (BuildContext context, AsyncSnapshot snapshot) {
                          return Slider(
                            min: 0,
                            max: widget.playVideoPlayer.getDurationMax().inMilliseconds.toDouble(),
                            value: widget.playVideoPlayer.getDurationPosition().inMilliseconds.toDouble(),
                            onChanged: (double value) {
                              setState(() {
                                widget.playVideoPlayer.seek(Duration(milliseconds: value.toInt()));
                              });
                            },
                          );
                        },
                      ),
                    ),
                    InkWell(
                      onTap: () {},
                      child: const Icon(
                        Icons.fullscreen,
                        color: Colors.white,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ),
      );
    }
    return child;
  }
}
